;;; abadguy.lsp -- code in this module concocts events relating to the
;;; villians of the adventure world. :ts=2
;;; Author -- John Bruno Hare.   Shanghai, 1988.
;;; (c) Copyright 1988 John Bruno Hare, all rights reserved.

(defun init-badguys()
	(setf (room-things (aref *rooms* *treasure-room*)) (gensym))
	(setq *random-state* (make-random-state t) ; use very random seed 
		*found-treasure* nil
		*under-attack* nil
		*ndwarves* 5
		*been-robbed* nil
		*threw-axe*	nil
		*pirate-probability* 30
		*dwarf-probability* 20
		*adventurer-strength* 24
		*hit-points* 24			; adventurer's hit points
		*dwarf-hit-points* 10
		*dwarf-strength* 10
		*treasure-chest* (room-things (aref *rooms* *treasure-room*)))
		(setq foo (random 55 *random-state*))		; kickstart random numbers
		)

; roll a dice with sides sides (0..sides-1)
(defun dice(sides)
	(random sides))

; things dwarves do
(defun dwarf-events()
	(if *under-attack*
		(let ((hit (dice 4)))
			(format t "The dwarf throws an little knife at you~%")
			(setq *hit-points* (- *hit-points* hit))
			(cond 
				((= hit 0)
					(format t "The knife misses!~%"))
				((= hit 1)
					(format t "The knife barely misses!~%"))
				((= hit 2)
					(format t "The knife scratches your skin! It really stings!~%"))
				((= hit 3)
					(format t "The knife makes a direct hit! Yow!~%")))
			(if (minusp *hit-points*)
				(return-from dwarf-events nil)
			(return-from dwarf-events t)))
	; else not under attack
	(progn
		(regenerate-adventurer)		; not under attack, regenerate
		(if (and (not (zerop *ndwarves*))(>= *cur-room-num* *on-the-ledge*))
			(if (zerop (dice *dwarf-probability*))
				(if (not *threw-axe*)		; first attack
					(progn
							(format t
"a dwarf ducks around a corner, curses, and throws an axe at you.
The axe misses.~%")
							(drop-in-room (aref *rooms* *cur-room-num*) 'axe)
							(setq *threw-axe* t))
				; subsequent attacks start on next call to this function
				(progn
					(format t "There is a threatening dwarf in the room with you!~%")
					(setq *under-attack* t *dwarf-hit-points* *dwarf-strength*)))))
		t)))

(defun regenerate-adventurer()
	(if (< *hit-points* *adventurer-strength*)
		(setq *hit-points* (+ *hit-points* (random 6)))))

; things that you do to the dwarf
(defun attack-dwarf()
	(if (not *under-attack*)
			(format t "There's no dwarf here!~%")
		(if (not (get *adventurer* 'axe))
				(format t "You don't have the magic axe~%")
			(let ((hit (dice 4)))
				(format t "You throw the magic axe at the dwarf!~%")
				(if (minusp (setq *dwarf-hit-points* (- *dwarf-hit-points* hit)))
					(progn
						(format t "the dwarf dissapears in a puff of greasy smoke!~%")
						(setq *ndwarves* (1- *ndwarves*));another one bites the dust
						(setq *under-attack* nil)
							(return-from attack-dwarf t))
					(cond 
						((= hit 0)
							(format t "The magic axe misses the dwarf!~%"))
						((= hit 1)
							(format t "The magic axe barely misses the dwarf!~%"))
						((= hit 2)
							(format t "The magic axe makes a good hit on the dwarf!~%"))
						((= hit 3)
							(format t
								"The magic axe makes a direct hit on the dwarf!~%"))))))))

(defun pirate-events()
		(if (and
					(not *under-attack*)	; give poor adventurer break if fighting dwarf
					(>= *cur-room-num* *on-the-ledge*))
			(if (zerop(dice *pirate-probability*))
				(let ((props (symbol-plist *adventurer*)))
					(if (null props)
							(format t "you hear a faint rustling noise behind you.~%")
						(progn
							(setq *been-robbed* t)
							(format t
"The pirate jumps out behind you!  Har Har, he laughs,
I'll just take this all loot and hide it away in the maze!~%")
							(do* ((do-props (copy-list props) (cdr do-props))
										(loot (car do-props)(car do-props)))
									((null do-props))
										(if (numberp (gethash loot *scores*))
											(progn
												(setf (get *treasure-chest* loot) t)
												(remprop *adventurer* loot)
												(setq do-props (cdr do-props)))))))))))
