;;; ainit.lsp
;;; initialization of the knowledge base of adventure.
;;; :ts=2
;;; Author -- John Bruno Hare.   Shanghai, 1988.
;;; (c) Copyright 1988 John Bruno Hare, all rights reserved.

; this function is used to set up the :things field.
; It returns a gensym with property list containing
; everything on list props each with the property t
(defun make-inventory(props)
	(let ((sym (gensym)))
		(dolist (x props)
			(setf (get sym x) 't))
		sym))

(defun init-advent-command-hash-table()
	(setq *advent-cmds* (make-hash-table :size 253))
	(puthash 'look *advent-cmds* #'advent-look-cmd)
	(puthash 'describe *advent-cmds* #'advent-describe-cmd)
	(puthash 'get *advent-cmds* #'advent-get-cmd)
	(puthash 'inventory *advent-cmds* #'print-inventory)
	(puthash 'drop *advent-cmds* #'advent-drop-cmd)
	(puthash 'unlock *advent-cmds* #'try-unlock)
	(puthash 'light *advent-cmds* #'try-light)
	(puthash 'jump *advent-cmds* #'try-jump)
	(puthash 'wave *advent-cmds* #'try-wave)
	(puthash 'read *advent-cmds* #'try-read)
	(puthash 'kill *advent-cmds* #'try-kill)
	(puthash 'end *advent-cmds* #'end-game)
	(puthash 'throw *advent-cmds* #'advent-throw-cmd)
	(puthash 'score *advent-cmds* #'advent-score-cmd)
	(puthash 'wait *advent-cmds* #'advent-wait-cmd))

; initialize the adventure world
(defun init-advent()
	(setq
		; database of rooms -- later the array *rooms* is
		; initialized to point to each room in this list
		*room-list*
		'(
			; room 0
			(make-room
				:short-desc "you are at the end of the road"
				:long-desc
"You are standing in front of a small house. A
stream of water is flowing from an open doorway."
				:paths '(north 1 in 1 south 2 down 2))

			; room 1
			(make-room
				:short-desc "you are in the well house"
				:long-desc
"You are in the well house.  Cold, clear, water
pours out of a two inch hole in the floor and tumbles
down a trough out the door."
				:paths '(south 0 out 0 xyzzy 7 y2 17)
				:things (make-inventory '(lantern key)))

			; room 2
			(make-room
				:short-desc "beside the stream"
				:long-desc
"You are walking beside the stream.  It flows
south into a gully.   All around you are dense woods."
				:paths '(north 0 south 3 down 3))

			; room 3
			(make-room
				:short-desc "at the grate"
				:long-desc
"You are standing in front of a round steel grate which
is set into the ground in the gully.  The stream flows
on from here south.  The walls of the gully narrow."
				:paths '(north 2 south 4 up 2 down 5))

			; room 4
			(make-room
				:short-desc "dead end"
				:long-desc
"The stream rushes into a slit in the rock.  There is no
way out of here except north."
				:paths '(north 3 up 4))

			; room 5
			(make-room
				:short-desc "under the grate"
				:long-desc
"You are standing at the low end of a long passage which
stretches west into darkness.  A gentle, cool breeze
is blowing from the interior of the cave."
				:paths '(west 6 up 3 east 3))

			; room 6
			(make-room
				:short-desc "cobble room"
				:long-desc
"The floor of this room is filled with many small polished
pebbles.  The passage runs east and west here. There is a
scrawled inscription on the wall:

		magic word XYZZY"
				:paths '(east 5 west 7 up 5 down 7)
				:things (make-inventory '(rod)))

			; room 7
			(make-room
				:short-desc "bird room"
				:long-desc
"This room has rough hewn columns made of some brassy metal.
The floor is covered with newspapers.  The newspaper is the
Froon Mercury News."
				:things (make-inventory '(bird newspaper))
				:paths '(west 8 down 8 east 6 up 6 xyzzy 1))

			; room 8
			(make-room
				:short-desc "top of the stairs"
				:long-desc
"This is the top of a long spiral staircase which descends
into the depths of the earth.  A sulphurous smell
emerges from below.  The breeze strengthens."
				:things (make-inventory '(cage))
				:paths '(east 7 up 7 down 9))

			; room 9
			(make-room
				:short-desc "on the ledge"
				:long-desc
"This is a narrow ledge.  To the east is the staircase.
A little alcove is to the north.   The crevasse to the west
is far too wide to jump."
				:paths '(up 8 east 8 down 11 north 10))

			; room 10
			(make-room
				:short-desc "alcove"
				:long-desc
"This is a shadowy little alcove.  There is a crude sign
on the wall -- you won't get it up the stairs."
				:paths '(out 9 south 9 sapiens 1)
				:things (make-inventory '(gold)))

			; room 11
			(make-room
				:short-desc "hall of mtn. king"
				:long-desc		; changes after battle
"This is the hall of the mountain king.   The stairs are
to the east.  The room is circular and the dim, high ceiling
is domed and covered with tesselated triangles.
An enormous fierce snake bars your path."
				:paths '(up 9 east 9 south 12 north 13 sw 20)
				:things (make-inventory '(snake cactus)))

			; room 12
			(make-room
				:short-desc "silver room"
				:long-desc
"This is a small chamber off the south side of the hall
of the mountain king."
				:things  (make-inventory '(silver))
				:paths '(out 11 north 11))

			; room 13
			(make-room
				:short-desc "coin room"
				:long-desc
"This is an anteroom off the north of the hall
of the mountain king.   There are many shelves
here."
				:things (make-inventory '(coin))
				:paths '(out 11 south 11 north 14))

			; room 14
			(make-room
				:short-desc "crossroads"
				:long-desc
"This a long north-south passage hewen from the rock.
There is a hole here leading down to a cooridor which
runs east-west."
				:paths '(north 15 south 13 down 17))

			; room 15
			(make-room
				:short-desc "west end of gallery"
				:long-desc
"This is the west end of a long gallery which
stretches far off to the east.   The walls are
smudged with soot except for rectangular areas
at intervals where portraits once hung."
				:paths '(down 14 east 16 north 18 south 22))

			; room 16
			(make-room
				:short-desc "east end of gallery"
				:long-desc
"The gallery abruptly ends at the edge of
a jagged crevasse whose bottom is invisible in the inky
depths below.   The sound of rushing water
can be heard below."
				:paths '(west 15 north 18))

			; room 17
			(make-room
				:short-desc "you are at y2"
				:long-desc
"This is a non-descript place in long concrete corridor.
A stenciled sign on the wall reads Y2.  Far away in the west
there is a strange purple illumination.  Quite a ways
to the east the passage twists abruptly."
				:paths '(up 14 west 21 east 22))

			; room 18
			(make-room
				:short-desc "side gallery"
				:long-desc
"You have crawled into a narrow passage
which apparently parallels the main portrait gallery."
				:things (make-inventory '(diamond))
				:paths '(sw 15 se 16))

			; room 19
			(make-room
				:short-desc "on the bridge"
				:long-desc
"The magic bridge spans the crevasse.  The bridge appears to
be made of marble and has intricate abstract carvings."
				; paths added later.
			)

			; room 20
			(make-room
				:short-desc "dragon room"
				:long-desc
				; the description changes after the battle.
"This is the dragon room.   A large green dragon is here,
resting on an enormous persian rug.  The dragons' breath
is scalding and smells like rotten eggs.  The leathery wings
flick back and forth lazily."
				:paths '(east 11 west 21)
				:things (make-inventory '(rug)))

			; room 21
			(make-room
				:short-desc "star sapphire room"
				:long-desc
"This is the star sapphire room.  It is lit by a shimmering
blue-purple light.   There is a large marble pedestal in the middle
of the chamber.  The walls of the chamber are shrouded in
a hazy vapor which conceals the exits (if any)."
				:paths '(sapiens 0)
				:things (make-inventory '(sapphire)))

			;;; the infamous maze of twisty passages

			; room 22
			(make-room
				:short-desc "in the maze"
				:long-desc "you are in a maze of twisty passages, all alike"
				:paths '(east 30 north 23 west 22))

			; room 23
			(make-room
				:short-desc "in the maze"
				:long-desc
					"you are in a little maze of twisty passages, all alike"
				:paths '(east 24 sw 29 west 23))

			; room 24
			(make-room
				:short-desc "in the maze"
				:long-desc
					"you are in a twisty maze of little passages, all alike"
				:paths '(north 30 east 29 south 28 west 25))

			; room 25
			(make-room
				:short-desc "in the maze"
				:long-desc "you are in a maze of twisty passages, all alike"
				:paths '(south 26 west 27))

			; room 26
			(make-room
				:short-desc "above the cobble room"
				:long-desc
					"This is the room where the pirate keeps his treasure."
				:paths '(down 6))

			; room 27
			(make-room
				:short-desc "in the maze"
				:long-desc
					"you are in a maze of twisty little passages, all alike"
				:paths '(north 27 west 23 se 31))

			; room 28
			(make-room
				:short-desc "in the maze"
				:long-desc "you are in a maze of twisty passages, all alike"
				:paths '(north 28 down 30))

			; room 29
			(make-room
				:short-desc "in the maze"
				:long-desc
					"you are in a little twisty maze of passages, all alike"
				:paths '(north 29 up 16))

			; room 30
			(make-room
				:short-desc "in the maze"
				:long-desc
					"you are in a twisty maze of little passages, all alike"
				:paths '(east 30 ne 22 north 23))

			; room 31
			(make-room
				:short-desc "vending machine"
				:long-desc
"There is a battery vending machine here.   The instructions
are all in Japanese, but it appears to take zorkmids.
There is spray painted Japanese graffiti all over the
walls and floor."
				:paths '(nw 27))
		)			;;; end of *room-list*
		; some random constants for interesting rooms
		*well-house*					1		; the house
		*above-the-grate*			3		; room above the grate
		*below-the-grate*			5		; room below the grate
		*start-cavern-room*		6		; the first room you need the lantern on in
		*top-of-stairs*				8
		*on-the-ledge*				9
		*crevasse-rooms*			'(9 16 19)	; don't jummmmmmp!
		*hall-of-mtn-king*		11
		*y2*									17
		*magic-bridge*				19
		*dragon-room*					20
		*sapphire-room*				21
		*treasure-room*				26
		*vending-machine-room*	31
		; hash table used to store treasure values for quick access
		*scores*							(make-hash-table :size 42)
	)
	; now stuff the array *rooms* with each room description
	(setq *rooms* (make-array (setq *nrooms* (list-length *room-list*))))
	(do
		((room *room-list* (cdr room))
		 (room-num 0 (1+ room-num)))
		((endp room))
			;(format t ">>> ~s~%" room)
			(setf (aref *rooms* room-num)(eval (car room))))

	; assign scores for each object which is a treasure
	(setq *hash-trace* t)
	(puthash 'sapphire *scores* 20)
	(puthash 'diamond *scores* 15)
	(puthash 'gold *scores* 12)
	(puthash 'silver *scores* 11)
	(puthash 'rug *scores* 5)
	(puthash 'cactus *scores* 4)
	(puthash 'coin *scores* 3)
	(puthash 'newspaper *scores* 1)
	(setq *max-score* 71)		; N.B. this number is maintained manually
	(setq
		*dir-verbs*					; words which mean motion (magical or otherwise)
			'(n s e w nw ne se sw u d north south east west 
				up down in out sapiens xyzzy y2)
		*dir-verbs-ht*
			(make-hash-table :size 67))
	(dolist (x *dir-verbs*)
		(puthash x *dir-verbs-ht* t))

	; assign descriptions for each object which can be picked up
	(setq *long-desc-map* (make-hash-table :size 42))
	(set-obj-long-desc 'axe
"This is a crude but effective axe of dwarvish manufacture.")
	(set-obj-long-desc 'bird
"A listless budgie.  But it's _not_ dead -- merely pining for the fiords")
	(set-obj-long-desc 'lantern
"A rusty battery powered lantern.")
	(set-obj-long-desc 'key "The key is intricately wrought.")
	(set-obj-long-desc 'newspaper
"This is just some old yellowing newspaper.  This news is very out of date.")
	(set-obj-long-desc 'rod
"The rod hums with a faint and slightly irritating vibration.   There
is a five pointed star in a circle on the end of it.  The rod is exactly
3.14159 bloits long. It is made of some dark heavy metal-like substance")
	(set-obj-long-desc 'cage
"The cage is made of wicker and has a little label on the bottom:

              made in Shanghai, P.R. China")
	(set-obj-long-desc 'gold
"This is a gold brick.   It's shiny and heavy and expensive.
Stamped on the top is a little seal which reads 256Kb drams.
The brick was manufactured in a mythical town named
Cambridgemass-under-Hill.")
	(set-obj-long-desc 'silver
"This is a pretty hefty bar of silver.  It is slightly tarnished with age.")
	(set-obj-long-desc 'coin
"This is an ancient coin which has a Bruin on one side and
the picture of a eunuch on the other.   The coin's value is
42 zorkmids.")
	(set-obj-long-desc 'diamond "The diamond is enormous and smeared with mud.")
	(set-obj-long-desc 'cactus "The cactus is green and prickly")
	(set-obj-long-desc 'rug "Atchooo! This rug is dusty! And reeks of dragon.")
	(set-obj-long-desc 'sapphire
"This is a fabulous sapphire, the size of a plover's egg.
In the depths of the sapphire can be seen a flickering star with
a hexagonal halo which shifts back and forth as you heft the gem
in your hands.")

; set up short descriptions for objects that 
; can be mentioned in room inventories
	(setq *short-desc-map* (make-hash-table :size 42))
	(set-obj-short-desc 'key "key")
	(set-obj-short-desc 'axe "axe")
	(set-obj-short-desc 'lantern "rusty lantern")
	(set-obj-short-desc 'gold "gold brick")
	(set-obj-short-desc 'silver "silver bar")
	(set-obj-short-desc 'rod "rod")
	(set-obj-short-desc 'rug "persian rug")
	(set-obj-short-desc 'bird "budgie")
	(set-obj-short-desc 'cage "wicker cage")
	(set-obj-short-desc 'coin "platinum coin")
	(set-obj-short-desc 'diamond "mud-encrusted diamond")
	(set-obj-short-desc 'cactus "potted cactus")
	(set-obj-short-desc 'newspaper "tattered newspaper")
	(set-obj-short-desc 'snake "fierce serpent")
	(set-obj-short-desc 'sapphire "star sapphire")
; set up map of synonyms for various words
	(setq *synonyms* (make-hash-table :size 99))
	(puthash 'n *synonyms* 'north)
	(puthash 's *synonyms* 'south)
	(puthash 'e *synonyms* 'east)
	(puthash 'w *synonyms* 'west)
	(puthash 'u *synonyms* 'up)
	(puthash 'd *synonyms* 'down)
	(puthash 'l *synonyms* 'look)
	(puthash 'desc *synonyms* 'describe)
	(puthash 'inspect *synonyms* 'describe)
	(puthash 'examine *synonyms* 'describe)
	(puthash 'i *synonyms* 'inventory)
	(puthash 'invent *synonyms* 'inventory)
	(puthash 'lamp *synonyms* 'lantern)
	(puthash 'brick *synonyms* 'gold)
	(puthash 'bar *synonyms* 'silver)
	(puthash 'budgie *synonyms* 'bird)
	(puthash 'paper *synonyms* 'newspaper)
	(puthash 'wand *synonyms* 'rod)
	(puthash 'staff *synonyms* 'rod)
	(puthash 'attack *synonyms* 'kill)
	(puthash 'toss *synonyms* 'throw)
	(puthash 'open *synonyms* 'unlock)
	(puthash 'walk *synonyms* 'go)
	(puthash 'run *synonyms* 'go)
	(puthash 'take *synonyms* 'get)
	(puthash 'grab *synonyms* 'get)
	(puthash 'leave *synonyms* 'drop)
	(puthash 'everything *synonyms* 'all)
	(puthash 'todo *synonyms* 'all)
	(init-badguys)
)			;;; end of init-advent
