;;; aobject.lsp  this handles the objects in the cave, including objects :ts=2
;;; with particular properties.
;;; Author -- John Bruno Hare.   Shanghai, 1988.
;;; (c) Copyright 1988 John Bruno Hare, all rights reserved.

;;; much of this code special cases certain objects and situations.
;;; no fair peeking if you haven't solved the riddles yet!

; take object from room: advise the adventurer
; if it is not there for the taking or if they already have it.
(defun get-item(object room)
	(if (not (symbolp object))
		(format t "get what?~%")
	(if (get *adventurer* object)
		(format t "You're already carrying the ~a.~%" (get-obj-short-desc object))
 	(let ((room-inventory (room-things room)))
		(if (get room-inventory object)
			(cond 
				((eq object 'bird) (get-bird room))
				((eq object 'snake)
						(format t "The snake slithers out of reach.~%"))
				((and (eq object 'rug)(not *killed-dragon*))
					(format t "the dragon is sitting on the rug.~%"))
				(t
			 	(progn
						(remprop room-inventory object)
						(setf (get *adventurer* object) t)
						(format t "You now have a ~a~%" (get-obj-short-desc object)))))
		(format t "there's no ~a here~%" (get-obj-short-desc object)))))))

; drop object in room.  If the adventurer doesn't have the
; object, let them know.
(defun drop-item(object room)
	(if (not (symbolp object))
		(format t "drop what?~%")
	(if	(not (get *adventurer* object))
			(format t "you're not carrying a ~a~%" (get-obj-short-desc object))
		(progn
			; if dropping object in well house, compute
			; max score on the fly and see if we should go
			; on to the end game
		 (if (and *been-robbed* *found-treasure* (eq *cur-room-num* *well-house*))
				(let ((this-score (gethash object *scores*)))
					(if (numberp this-score)
							(if (<= *max-score* (+ (get-score) this-score))
								(progn
									(remprop *adventurer* object)
									(setf (get (room-things room) object) t)
									(end-game t t))))))
		 ; otherwise fall through to...
		 (cond
			; bird and snake
			((and (eq *cur-room-num* *hall-of-mtn-king*)
								(eq object 'bird)
								(get (room-things room) 'snake))
							(format t
"The bird puts on an unexpected burst of aggression
and, talons extended, drives the snake off~%")
							(remprop *adventurer* 'bird)
							(remprop (room-things room) 'snake)
							(setf (get (room-things room) 'bird) t)
							(setf (room-long-desc room)
"This is the hall of the mountain king.   The stairs are
to the east.  The room is circular and the dim, high ceiling
is domed and covered with tesselated triangles.
There are exits in several directions."))

			; if carrying bird and cage, dropping cage, drop both.
			((and (eq object 'cage)(get *adventurer* 'bird))
						(setf (get (room-things room) 'bird) t)
						(setf (get (room-things room) 'cage) t)
						(remprop *adventurer* 'bird)
						(remprop *adventurer* 'cage)
						(format t
"You set down the birdcage.   The bird deftly picks the lock and
hops out of the cage.~%"))

			; vending machine
			((and (eq *cur-room-num* *vending-machine-room*)
					(eq object 'coin))
						(progn
							(format t
"You drop the coin in the slot. The batteries fall into
the hopper: you insert them into the lantern and it burns
with renewed light.~%")
								(remprop *adventurer* 'coin)
								(setq  *lantern-moves* 1000 *lantern-half-life* 500)))

		;;; normal drop
			(t 
				(drop-in-room room object)
				(remprop *adventurer* object)
				(format t "dropped a ~a~%" (get-obj-short-desc object))))))))

; primitive to drop an object in room.  NOTE: This does not
; remove it from anything (particularly the adventurer).
(defun drop-in-room(room object)
	(let ((room-inventory (room-things room)))
			(if (null room-inventory)
				; this room has never had anything in it.
					; gensym a symbol for its inventory
					(progn
						(setq room-inventory (gensym))
						(setf (room-things room) room-inventory)))
			(setf (get room-inventory object) t)))

; unlock grate
(defun try-unlock(object y)
	(if (not (symbolp object))
		(format t "Unlock what?~%")
	(if (not (eq object 'grate))
			(format t "you can't unlock the ~a~%" (get-obj-short-desc object))
		(if (not (get *adventurer* 'key))
				(format t "you don't have the key.~%")
			(if (not (eq *cur-room-num* *above-the-grate*))
					(format t "there's nothing to unlock here~%")
				(if (not *grate-locked*)
						(format t "the grate is already unlocked~%")
					(progn
						(setq *grate-locked* nil)
						(format t "You use the key to unlock the grate.~%"))))))))

; light the lantern
(defun try-light(object y)
	(if (not (symbolp object))
		(format t "light what?~%")
	(if (not (eq object 'lantern))
			(format t "you can't light the ~a~%" (get-obj-short-desc object))
		(if (not (get *adventurer* 'lantern))
				(format t "you don't have the rusty lantern~%")
			(if *lantern-lit*
					(format t "the lantern is already lit~%")
				(if (not (plusp *lantern-moves*))
						(format t "the batteries are completely dead.~%")
					(progn
						(setq *lantern-lit* t)
						(format t "<*click*>...  The lantern is now lit.~%"))))))))

; catch the bird
(defun get-bird(room)
	(if (get *adventurer* 'bird)
			(format t "you already have the bird.~%")
		(if (not (get (room-things room) 'bird))
			(format t "what bird?~%")
		(if (get *adventurer* 'rod)
			(format t "the bird is terrified and flutters out of your reach.~%")
			(if (not (get *adventurer* 'cage))
				(format t "you have nothing to put the bird in.~%")
			(progn
				(format t
						"the bird hops into the cage and starts to sing cheerfully.~%")
						(setf (get *adventurer* 'bird) t)
						(remprop (room-things room) 'bird)))))))

; kill something
(defun try-kill(object &optional y)
	(if (not (or (symbolp object)(eq object 'dragon)(eq object 'dwarf)))
			(format t "Pointless violence is not considered acceptable behavior.~%")
		; we know that the dragon or a dwarf is the object now.
		(if (eq object 'dwarf)
				(attack-dwarf)
			(if (not (eq *cur-room-num* *dragon-room*))
					(format t "The dragon is nowhere to be seen~%")
				(if *killed-dragon*
						(format t "The dragon is already dead.~%")
					(if (y-or-n-p "with what? your bare hands? ")		; huh?
							(progn
								; gosh!
								(format t			
			"You have killed a dragon with your bare hands! Amazing...~%")
								(setq *killed-dragon* t)
								(setf (room-long-desc (aref *rooms* *dragon-room*))
"This is the dragon room.   A large dead green dragon is here.
There is a curious flickering blue light coming from a crack in
the west wall behind the dragons carcass."))))))))


(defun try-wave(object y)
	(if (not (symbolp object))
		(format t "hi, mom!~%")
	(if (eq object 'rod)
		(if (member *cur-room-num* *crevasse-rooms*)
			(let ((east-side (first *crevasse-rooms*))
						(west-side (second *crevasse-rooms*)))
				(format t "a bridge magically spans the crevasse!~%")
				(link-rooms 'west east-side *magic-bridge*)
				(link-rooms 'east *magic-bridge* east-side)
				(link-rooms 'east west-side *magic-bridge*)
				(link-rooms 'west *magic-bridge* west-side))
			(format t "nothing happens.~%"))
		(format t "hi, mom!~%"))))

(defun try-read(object y)
	(if (not (symbolp object))
		(format t "Read what?~%")
	(if (eq object 'newspaper)
			(format t
"The only article of interest is a report on the software industry
in Pheebor: Implementors Go On Feature Adding Rampage, Jungle
Ride Grinds to Halt.~%")
			(format t "There's nothing on it to read~%"))))

; print the adventurers inventory
(defun print-inventory(object)
	(let ((props (symbol-plist *adventurer*)))
		(if (null props)
				(format t "you are not carrying anything~%")
			(do ((do-props props (cdr do-props)))
					((null do-props))
						(format t "you have a ~a~%" (get-obj-short-desc (car do-props)))
						(setq do-props (cdr do-props))))))


; assign an objects long description
(defun set-obj-long-desc(object description)
	(puthash object *long-desc-map* description))
; return the short description for the given object)
(defun get-obj-long-desc(object)
	(let ((desc (gethash object *long-desc-map*)))
		(if desc desc object)))

; assign an objects (short) description
(defun set-obj-short-desc(object description)
	(puthash object *short-desc-map* description))
; return the short description for the given object)
(defun get-obj-short-desc(object)
	(let ((desc (gethash object *short-desc-map*)))
		(if desc desc object)))
