;;; aplay.lsp  -- play adventure :ts=2
;;; Author -- John Bruno Hare.   Shanghai, 1988.
;;; (c) Copyright 1988 John Bruno Hare, all rights reserved.

(defun advent()
	(setq			; various inits which are needed to run the game 
		*cur-room-num* 0			; current node in network
	  *adventurer* (gensym)	; cons up a symbol for the adventurers properties
		*grate-locked* t			; the grate is locked
		*killed-dragon* nil 	; dragon is alive
		; these variables relate to the lantern
		*lantern-lit* nil			; is the lantern on
		*lantern-moves* 1000	; number of moves with lantern lit
		*lantern-half-life* 500 ; warn adventurer at this point
		)
	(if (not (catch 'top-level-advent
	 (let ((room (aref *rooms* *cur-room-num*))(grue-p nil))
		(loop
			; if we're in the cave, the lantern must be on
			(if (and *lantern-lit* (= *lantern-moves* *lantern-half-life*))
				(format t
"The lamp is getting dimmer.  By the way, there's a rumor that there's
a battery vending machine somewhere in the cave.  Better
take some money."))
			; turn off the lantern if time is up
			(if (and *lantern-lit* (zerop (decf *lantern-moves*)))
					(progn
						(format t "the lantern goes phhht...~%")
						(setq *lantern-lit* nil)))
			; if this room is not lit, adventurer is in trouble.
			(if (room-in-light-p room)
					(progn
						(setq grue-p nil)
						(if (not (dwarf-events))
								(progn
									(format t "Oh NO! You have been killed by a dwarf.~%")
									(format t "your score was ~d~%" (get-score))
									(return-from advent nil))))
				(if grue-p
					(progn
						(format t "argggh! you have been eaten by a grue!~%")
						(format t "your score was ~d~%" (get-score))
						(return-from advent nil))
					; warn if in the dark
					(progn
						(setq grue-p t)
						(format t
"You are in pitch darkness.  If you proceed any further without
light you might be attacked by a grue.~%"))))

			; get the command and act on it
			(setq room (aref *rooms* *cur-room-num*))
			(if (room-in-light-p room)
				(progn
					(print-room room)
					; mark this room as visited if not already so
					(if (not (room-visited-p room))
						(setf (room-visited-p room) t))
					(print-room-contents room)))
			(format t "> ")	; output prompt
			(let ((cmd-list (getcmd)))
				(if (not (docmd cmd-list room))
					(if (y-or-n-p "really leave? ")
						(progn
							(format t "your score was ~d~%" (get-score))
							(return-from advent t)))))
			(setq room (aref *rooms* *cur-room-num*))
			(if (room-in-light-p room)
					(pirate-events))))))
		; catch return value determines which of these endgames
		; resolutions occur.
			(progn
				(format t "you wimped out in the end game. [score=~s]~%"
							(get-score)) nil)
		(progn
			(format t "you are a total winner. [score=~s]~%"
						(get-score)) t)))

(format t "initializing advent~%")
(init-advent)
(format t "initializing advent command hash tables~%")
(init-advent-command-hash-table)
(format t "game begins~%")
(advent)
