;;; aroom.lsp -- code specifically about rooms :ts=2
;;; Author -- John Bruno Hare.   Shanghai, 1988.
;;; (c) Copyright 1988 John Bruno Hare, all rights reserved.

; print the short or long description of this room depending on
; whether or not we have been there yet.
(defun print-room(room)
	(if (room-visited-p room)
		(format t "~a~%" (room-short-desc room))
	(format t "~a~%" (room-long-desc room))))

; print the properties of this room
(defun print-room-contents(room)
	(do ((props (symbol-plist (room-things room))(cdr props)))
			((null props))
				(format t "there is a ~a here~%" (get-obj-short-desc (car props)))
				(setq props (cdr props))))

; explicitly link room with number room-0 to room with number room-1
; in the given direction.  Note that this is a one way link:  To
; establish a transitive relation, you have to call this twice.
; For safety's sake this conses a bit more than it should, so this
; should only be used for cases where the link cannot be pre-established.
(defun link-rooms (direction room-0 room-1)
	(let* ((room (aref *rooms* room-0))
				 (pathls (copy-list (room-paths room))))
		(revpush direction pathls)
		(revpush room-1 pathls)
		(setf (room-paths room) pathls)))

; is this room lit?
(defun room-in-light-p(room)
	(or (< *cur-room-num* *start-cavern-room*)
			(and *lantern-lit*
						(or (get *adventurer* 'lantern)
								(get (room-things room) 'lantern)))
			(eq *cur-room-num* *sapphire-room*)))
