;;; avocab.lsp -- the 'natural language' interface to the adventure program.
;;; :ts=2
;;; Author -- John Bruno Hare.   Shanghai, 1988.
;;; (c) Copyright 1988 John Bruno Hare, all rights reserved.


; act upon a command
(defun docmd(cmd room)
	;(format t ">>> command is ~s~% room=~a~%" cmd room)
	(if (null cmd)
		(return-from docmd nil)
	  (if (consp cmd)
		  (if (> (length cmd) 2)
			  (format t "command must be one or two words please~%")
		  (let 
				  ((verb (first cmd))
				   (object (second cmd))
				   (is-motion-verb nil))
			  (setq verb (get-true-name verb))
			  (format t ".")
			  (setq object (get-true-name object))
			  (format t ".")
			  (cond
				  ; does this require motion to another room
				  ; is this a directional "verb"?
				  ((or (gethash verb *dir-verbs-ht*)
							  (setq is-motion-verb (eq verb 'go)))
					  (if is-motion-verb		; use second word instead
						  (setq verb object))
					  (let ((paths-out-from-this-room (room-paths room))(path-out-mem-ls))
						  (setq path-out-mem-ls (member verb paths-out-from-this-room))
						  (if (consp path-out-mem-ls)
								  (setq *cur-room-num*
									  (move-adventurer *cur-room-num* (second path-out-mem-ls)))
							  ; we know this is a direction but not a possible direction
							  (if (magic-motion-verb-p verb)
								  (format t "nothing happens...~%")
							  (format t "sorry, no way ~s~%" verb)))))
				  (t
					  (let ((fn (gethash verb *advent-cmds*)))
						  (if fn
								  (funcall fn object room)
							  (format t "sorry, I don't recognize that request~%")))))))))
	  cmd)		; returns cmd
(defun advent-look-cmd(object y) 
	(if (not (null object))
		(if (get *adventurer* object)
					(format t "~a~%" (get-obj-long-desc object))
				(format t "I'm afraid you're not holding a ~a~%"
													(get-obj-short-desc object)))
			(format t "~a~%" (room-long-desc (aref *rooms* *cur-room-num*)))))
(defun advent-describe-cmd(object y)
	(if (get *adventurer* object)
			(format t "~a~%" (get-obj-long-desc object))
		(format t "I'm afraid you're not holding a ~a~%"
							(get-obj-short-desc object))))
(defun advent-drop-cmd(object room)
	(if (eq object 'all)
		(dolist (item (copy-list (symbol-plist *adventurer*)))
			(if (not (eq t item))
					(drop-item item room)))
				(drop-item object room)))
(defun advent-get-cmd(object room)
	(if (eq object 'all)
			(dolist (item (copy-list (symbol-plist (room-things room))))
							(if (not (eq t item))
								(get-item item room)))
			(get-item object room)))
(defun advent-throw-cmd(object room)
		(if (eq object 'axe)		; magic axe never leaves our hands
				(try-kill 'dwarf)
			(drop-item object room)))
(defun advent-score-cmd(x y)
		(format t "the score is now ~d~%" (get-score)))
(defun advent-wait-cmd(x y)
	(format t "...time passes~%"))
; this returns the total score
(defun get-score()
	(let ((score 0)(this-score))
		(dolist (treasure (symbol-plist (room-things (aref *rooms* *well-house*))))
			#|(format t "checking ~s: score for this is ~s~%" 
					treasure (gethash treasure *scores*))|#
			(if (numberp (setq this-score (gethash treasure *scores*)))
				(setq score (+ score this-score))))
					score))

; get the 'true name' for this word.  Return word if no
; particular true-name
(defun get-true-name(word)
	(if (symbolp word)
		(let ((true-name (gethash word *synonyms*)))
			(return-from get-true-name (if true-name true-name word))))
		word)

; is this an attack verb?
(defun attack-verb(verb)
	(member verb '(attack kill)))

(defun magic-motion-verb-p (verb)
	(member verb '(xyzzy sapiens y2)))

; This function reads a command from the keyboard.
; The command is digested into a list. Specifically the string
; typed in is converted into a list of symbols (in uppercase).
; if the user typed quit or q, return nil.
(defun getcmd()
	(let ((s " "))
		(tagbody
tryagain
		(setq s (read-line))
		(if (equalp s "")(go tryagain)))
		(if (or (string-equal s "quit")(string-equal s "q"))
			nil
	   (string-to-list s))))
