; tower of hanoi solver. actual algorithm from Common LISP by Deborah Tatar
; (p. 108)		:ts=2
; display stuff by John Bruno Hare

(setq *max-disks* 10)

; this function from Tatar actually does the work
(defun tower-of-hanoi (n from to spare)
	(cond ((= n 1) (move-disk from to))
		(t (tower-of-hanoi (- n 1) from spare to)
			(move-disk from to)
			(tower-of-hanoi (- n 1) spare to from))))

; provide a graphic display of the solution
; prepare to display n disks
(defun setup-disks(n)
	(if (> n *max-disks*)
		(error "please select a number of disks less than ~d.~%" *max-disks*))
	(setq
			*move* 0
			*disks* n
			*a-rod* (make-array n)
			*b-rod* (make-array n)
			*c-rod* (make-array n)
			*a-disks* n
			*b-disks* 0
			*c-disks* 0
			*disk-top-row* 10
			*a-rod-col* 20 *b-rod-col* 40 *c-rod-col* 60
			*a-empty-disk-col* (- 20 n)
			*b-empty-disk-col* (- 40 n)
			*c-empty-disk-col* (- 60 n))
	(setq
			*a-rod-row* *disk-top-row*
			*b-rod-row* (+ *disk-top-row* *disks*)
			*c-rod-row* (+ *disk-top-row* *disks*))
	(setq *empty-disk* (make-string (* 2 n) :initial-element #\space))
	; initialize disks on the a rod
	(dotimes (i n)
				(setf (aref *a-rod* i)
					; use IBM block character to fill disk
					(make-string (* 2 (+ i 1)) :initial-element (character #xdb))))
	(init-print-rod))

; set up the rods on the A disk
(defun init-print-rod()
	(do ((i *a-disks* (- i 1)))
			((zerop i))
				(curpos (+ *disk-top-row* (- i 1)) (- *a-rod-col* i))
				(format t "~a" (aref *a-rod* (- i 1)))))

; remove the top disk from 'from': put it on 'to'
(defun show-move(from to)
	(setq *move* (+ 1 *move*))
	(case from
		(a
			(let ((disk-offset (- *disks* *a-disks*)))
				(setq disk-to-move (aref *a-rod* disk-offset))
				(setq *a-rod-row* (+ *disk-top-row* disk-offset))
				(curpos *a-rod-row* *a-empty-disk-col*)
				(format t "~a" *empty-disk*)
				(setq *a-disks* (1- *a-disks*))))
		(b
			(let ((disk-offset (- *disks* *b-disks*)))
				(setq disk-to-move (aref *b-rod* disk-offset))
				(setq *b-rod-row* (+ *disk-top-row* disk-offset))
				(curpos *b-rod-row* *b-empty-disk-col*)
				(format t "~a" *empty-disk*)
				(setq *b-disks* (1- *b-disks*))))
		(c
			(let ((disk-offset (- *disks* *c-disks*)))
				(setq disk-to-move (aref *c-rod* disk-offset))
				(setq *c-rod-row* (+ *disk-top-row* disk-offset))
				(curpos *c-rod-row* *c-empty-disk-col*)
				(format t "~a" *empty-disk*)
				(setq *c-disks* (1- *c-disks*))))
		(otherwise (error "show-move: 'from' disk not recognized")))
	(case to
		(a
			(setq *a-disks* (1+ *a-disks*))
			(let ((disk-offset (- *disks* *a-disks*)))
				(setq *a-rod-row* (+ *disk-top-row* disk-offset))
				(setf (aref *a-rod* disk-offset) disk-to-move)
				(curpos *a-rod-row* (- *a-rod-col* (/ (length disk-to-move) 2)))
				(format t "~a" disk-to-move)))
		(b
			(setq *b-disks* (1+ *b-disks*))
			(let ((disk-offset (- *disks* *b-disks*)))
				(setq *b-rod-row* (+ *disk-top-row* disk-offset))
				(setf (aref *b-rod* disk-offset) disk-to-move)
				(curpos *b-rod-row* (- *b-rod-col* (/ (length disk-to-move) 2)))
				(format t "~a" disk-to-move)))
		(c
			(setq *c-disks* (1+ *c-disks*))
			(let ((disk-offset (- *disks* *c-disks*)))
				(setq *c-rod-row* (+ *disk-top-row* disk-offset))
				(setf (aref *c-rod* disk-offset) disk-to-move)
				(curpos *c-rod-row* (- *c-rod-col* (/ (length disk-to-move) 2)))
				(format t "~a" disk-to-move)))
		(otherwise (error "show-move: 'to' disk not recognized")))
)

(defun move-disk(from to)
	(curpos 3 3)
	(format t "~d Move a disk from ~a to ~a." *move* from to)
	(show-move from to)
	(curpos 4 3)
	; optional- print number of disks on each rod
	;(format t "a=~d b=~d c=~d      " *a-disks* *b-disks* *c-disks*)
	)
(defun demo-hanoi()
    (cls)
    (loop
	    (setq *disks* (accept 'fixnum "how many disks? [ESC to quit]"))
	    (cls)
    	(if (not *disks*) return)
			(cls)
	    (setup-disks *disks*)
    	(tower-of-hanoi *disks* 'a 'b 'c)
			))
(demo-hanoi)
